import * as blurhash from "blurhash";
import sharp from "sharp";
function decode83(str: string) {
  return [...str].reduce(
    (value, char) =>
      value * 83 +
      "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz#$%*+,-.:;=?@[]^_{|}~" //
        .indexOf(char),
    0,
  );
}
function getSizeFromHash(hash: string) {
  const sizeFlag = decode83(hash[0]);
  const height = ((sizeFlag / 9) | 0) + 1;
  const width = (sizeFlag % 9) + 1;
  return { width, height };
}
export async function encodeBlurhashPng(hash: string) {
  const size = getSizeFromHash(hash);
  const data = blurhash.decode(hash, size.width, size.height);
  return (await sharp(data, {
    raw: { width: size.width, height: size.height, channels: 4 },
  }).png({ colors: size.width * size.height }).toBuffer());
}
export async function PrecomputedBlurhash({ hash }: { hash: string }) {
  const data = await encodeBlurhashPng(hash);
  return (
    <span
      style={`background:url("data:image/png;base64,${
        data.toString("base64url")
      }")0px 0px/100% auto;`}
    />
  );
}
